@@ -144,8 +144,7 @@ Result BinaryReaderLogging::OnFuncType(Index index,
                                        Type* param_types,
                                        Index result_count,
                                        Type* result_types) {
-  // TODO: switch to "OnFuncType"?
-  LOGF("OnType(index: %" PRIindex ", params: ", index);
+  LOGF("OnFuncType(index: %" PRIindex ", params: ", index);
   LogTypes(param_count, param_types);
   LOGF_NOINDENT(", results: ");
   LogTypes(result_count, result_types);
@@ -242,16 +241,16 @@ Result BinaryReaderLogging::OnImportGlobal(Index import_index,
                                  global_index, type, mutable_);
 }
 
-Result BinaryReaderLogging::OnImportEvent(Index import_index,
-                                          string_view module_name,
-                                          string_view field_name,
-                                          Index event_index,
-                                          Index sig_index) {
-  LOGF("OnImportEvent(import_index: %" PRIindex ", event_index: %" PRIindex
+Result BinaryReaderLogging::OnImportTag(Index import_index,
+                                        string_view module_name,
+                                        string_view field_name,
+                                        Index tag_index,
+                                        Index sig_index) {
+  LOGF("OnImportTag(import_index: %" PRIindex ", tag_index: %" PRIindex
        ", sig_index: %" PRIindex ")\n",
-       import_index, event_index, sig_index);
-  return reader_->OnImportEvent(import_index, module_name, field_name,
-                                event_index, sig_index);
+       import_index, tag_index, sig_index);
+  return reader_->OnImportTag(import_index, module_name, field_name, tag_index,
+                              sig_index);
 }
 
 Result BinaryReaderLogging::OnTable(Index index,
@@ -376,9 +375,12 @@ Result BinaryReaderLogging::OnLoopExpr(Type sig_type) {
   return reader_->OnLoopExpr(sig_type);
 }
 
-Result BinaryReaderLogging::OnSelectExpr(Type return_type) {
-  LOGF("OnSelectExpr(return_type: %s)\n", return_type.GetName());
-  return reader_->OnSelectExpr(return_type);
+Result BinaryReaderLogging::OnSelectExpr(Index result_count,
+                                         Type* result_types) {
+  LOGF("OnSelectExpr(return_type: ");
+  LogTypes(result_count, result_types);
+  LOGF_NOINDENT(")\n");
+  return reader_->OnSelectExpr(result_count, result_types);
 }
 
 Result BinaryReaderLogging::OnTryExpr(Type sig_type) {
@@ -612,14 +614,14 @@ Result BinaryReaderLogging::OnSectionSymbol(Index index,
   return reader_->OnSectionSymbol(index, flags, section_index);
 }
 
-Result BinaryReaderLogging::OnEventSymbol(Index index,
-                                          uint32_t flags,
-                                          string_view name,
-                                          Index event_index) {
-  LOGF("OnEventSymbol(name: " PRIstringview " flags: 0x%x index: %" PRIindex
-           ")\n",
-       WABT_PRINTF_STRING_VIEW_ARG(name), flags, event_index);
-  return reader_->OnEventSymbol(index, flags, name, event_index);
+Result BinaryReaderLogging::OnTagSymbol(Index index,
+                                        uint32_t flags,
+                                        string_view name,
+                                        Index tag_index) {
+  LOGF("OnTagSymbol(name: " PRIstringview " flags: 0x%x index: %" PRIindex
+       ")\n",
+       WABT_PRINTF_STRING_VIEW_ARG(name), flags, tag_index);
+  return reader_->OnTagSymbol(index, flags, name, tag_index);
 }
 
 Result BinaryReaderLogging::OnTableSymbol(Index index,
@@ -731,6 +733,15 @@ Result BinaryReaderLogging::OnComdatEntry(ComdatType kind, Index index) {
     return reader_->name(opcode, alignment_log2, offset);                 \
   }
 
+#define DEFINE_SIMD_LOAD_STORE_LANE_OPCODE(name)                             \
+  Result BinaryReaderLogging::name(Opcode opcode, Address alignment_log2,    \
+                                   Address offset, uint64_t value) {         \
+    LOGF(#name "(opcode: \"%s\" (%u), align log2: %" PRIaddress              \
+               ", offset: %" PRIaddress ", lane: %" PRIu64 ")\n",            \
+         opcode.GetName(), opcode.GetCode(), alignment_log2, offset, value); \
+    return reader_->name(opcode, alignment_log2, offset, value);             \
+  }
+
 #define DEFINE0(name)                  \
   Result BinaryReaderLogging::name() { \
     LOGF(#name "\n");                  \
@@ -788,13 +799,15 @@ DEFINE_LOAD_STORE_OPCODE(OnAtomicStoreExpr);
 DEFINE_LOAD_STORE_OPCODE(OnAtomicWaitExpr);
 DEFINE_INDEX_DESC(OnAtomicFenceExpr, "consistency_model");
 DEFINE_LOAD_STORE_OPCODE(OnAtomicNotifyExpr);
-DEFINE_INDEX_INDEX(OnBrOnExnExpr, "depth", "event_index");
 DEFINE_OPCODE(OnBinaryExpr)
 DEFINE_INDEX_DESC(OnCallExpr, "func_index")
 DEFINE_INDEX_INDEX(OnCallIndirectExpr, "sig_index", "table_index")
-DEFINE0(OnCatchExpr);
+DEFINE0(OnCallRefExpr)
+DEFINE_INDEX_DESC(OnCatchExpr, "tag_index");
+DEFINE0(OnCatchAllExpr);
 DEFINE_OPCODE(OnCompareExpr)
 DEFINE_OPCODE(OnConvertExpr)
+DEFINE_INDEX_DESC(OnDelegateExpr, "depth");
 DEFINE0(OnDropExpr)
 DEFINE0(OnElseExpr)
 DEFINE0(OnEndExpr)
@@ -822,17 +835,20 @@ DEFINE_INDEX(OnRefFuncExpr)
 DEFINE_TYPE(OnRefNullExpr)
 DEFINE0(OnRefIsNullExpr)
 DEFINE0(OnNopExpr)
-DEFINE0(OnRethrowExpr);
+DEFINE_INDEX_DESC(OnRethrowExpr, "depth");
 DEFINE_INDEX_DESC(OnReturnCallExpr, "func_index")
 
 DEFINE_INDEX_INDEX(OnReturnCallIndirectExpr, "sig_index", "table_index")
 DEFINE0(OnReturnExpr)
 DEFINE_LOAD_STORE_OPCODE(OnLoadSplatExpr);
+DEFINE_LOAD_STORE_OPCODE(OnLoadZeroExpr);
 DEFINE_LOAD_STORE_OPCODE(OnStoreExpr);
-DEFINE_INDEX_DESC(OnThrowExpr, "event_index")
+DEFINE_INDEX_DESC(OnThrowExpr, "tag_index")
 DEFINE0(OnUnreachableExpr)
 DEFINE_OPCODE(OnUnaryExpr)
 DEFINE_OPCODE(OnTernaryExpr)
+DEFINE_SIMD_LOAD_STORE_LANE_OPCODE(OnSimdLoadLaneExpr);
+DEFINE_SIMD_LOAD_STORE_LANE_OPCODE(OnSimdStoreLaneExpr);
 DEFINE_END(EndCodeSection)
 
 DEFINE_BEGIN(BeginElemSection)
@@ -882,10 +898,10 @@ DEFINE_INDEX(OnInitFunctionCount)
 DEFINE_INDEX(OnComdatCount)
 DEFINE_END(EndLinkingSection)
 
-DEFINE_BEGIN(BeginEventSection);
-DEFINE_INDEX(OnEventCount);
-DEFINE_INDEX_INDEX(OnEventType, "index", "sig_index")
-DEFINE_END(EndEventSection);
+DEFINE_BEGIN(BeginTagSection);
+DEFINE_INDEX(OnTagCount);
+DEFINE_INDEX_INDEX(OnTagType, "index", "sig_index")
+DEFINE_END(EndTagSection);
 
 // We don't need to log these (the individual opcodes are logged instead), but
 // we still need to forward the calls.
@@ -914,6 +930,12 @@ Result BinaryReaderLogging::OnOpcodeUint32Uint32(uint32_t value,
   return reader_->OnOpcodeUint32Uint32(value, value2);
 }
 
+Result BinaryReaderLogging::OnOpcodeUint32Uint32Uint32(uint32_t value,
+                                                       uint32_t value2,
+                                                       uint32_t value3) {
+  return reader_->OnOpcodeUint32Uint32Uint32(value, value2, value3);
+}
+
 Result BinaryReaderLogging::OnOpcodeUint64(uint64_t value) {
   return reader_->OnOpcodeUint64(value);
 }
