@@ -49,7 +49,7 @@ struct Label {
         used(used) {}
 
   bool HasValue() const {
-    return label_type != LabelType::Loop && !sig.empty();
+    return !sig.empty();
   }
 
   LabelType label_type;
@@ -163,6 +163,7 @@ class CWriter {
 
   static char MangleType(Type);
   static std::string MangleTypes(const TypeVector&);
+  static std::string MangleMultivalueTypes(const TypeVector&);
   static std::string MangleName(string_view);
   static std::string MangleFuncName(string_view,
                                     const TypeVector& param_types,
@@ -222,6 +223,7 @@ class CWriter {
   void WriteInitExpr(const ExprList&);
   std::string GenerateHeaderGuard() const;
   void WriteSourceTop();
+  void WriteMultivalueTypes();
   void WriteFuncTypes();
   void WriteImports();
   void WriteFuncDeclarations();
@@ -264,8 +266,11 @@ class CWriter {
   void Write(const UnaryExpr&);
   void Write(const TernaryExpr&);
   void Write(const SimdLaneOpExpr&);
+  void Write(const SimdLoadLaneExpr&);
+  void Write(const SimdStoreLaneExpr&);
   void Write(const SimdShuffleOpExpr&);
   void Write(const LoadSplatExpr&);
+  void Write(const LoadZeroExpr&);
 
   const WriteCOptions& options_;
   const Module* module_ = nullptr;
@@ -330,14 +335,12 @@ void CWriter::PushLabel(LabelType label_type,
                         const std::string& name,
                         const FuncSignature& sig,
                         bool used) {
-  // TODO(binji): Add multi-value support.
-  if ((label_type != LabelType::Func && sig.GetNumParams() != 0) ||
-      sig.GetNumResults() > 1) {
-    UNIMPLEMENTED("multi value support");
-  }
-
-  label_stack_.emplace_back(label_type, name, sig.result_types,
-                            type_stack_.size(), used);
+  if (label_type == LabelType::Loop)
+    label_stack_.emplace_back(label_type, name, sig.param_types,
+                              type_stack_.size(), used);
+  else
+    label_stack_.emplace_back(label_type, name, sig.result_types,
+                              type_stack_.size(), used);
 }
 
 const Label* CWriter::FindLabel(const Var& var) {
@@ -405,6 +408,16 @@ std::string CWriter::MangleTypes(const TypeVector& types) {
   return result;
 }
 
+// static
+std::string CWriter::MangleMultivalueTypes(const TypeVector& types) {
+  assert(types.size() >= 2);
+  std::string result = "wasm_multi_";
+  for (auto type : types) {
+    result += MangleType(type);
+  }
+  return result;
+}
+
 // static
 std::string CWriter::MangleName(string_view name) {
   const char kPrefix = 'Z';
@@ -616,11 +629,16 @@ void CWriter::Write(const Var& var) {
 void CWriter::Write(const GotoLabel& goto_label) {
   const Label* label = FindLabel(goto_label.var);
   if (label->HasValue()) {
-    assert(label->sig.size() == 1);
+    size_t amount = label->sig.size();
     assert(type_stack_.size() >= label->type_stack_size);
-    Index dst = type_stack_.size() - label->type_stack_size - 1;
-    if (dst != 0)
-      Write(StackVar(dst, label->sig[0]), " = ", StackVar(0), "; ");
+    assert(type_stack_.size() >= amount);
+    assert(type_stack_.size() - amount >= label->type_stack_size);
+    Index offset = type_stack_.size() - label->type_stack_size - amount;
+    if (offset != 0) {
+      for (Index i = 0; i < amount; ++i) {
+        Write(StackVar(amount - i - 1 + offset, label->sig[i]), " = ", StackVar(amount - i - 1), "; ");
+      }
+    }
   }
 
   if (goto_label.var.is_name()) {
@@ -693,10 +711,12 @@ void CWriter::Write(SignedType type) {
 }
 
 void CWriter::Write(const ResultType& rt) {
-  if (!rt.types.empty()) {
+  if (rt.types.empty()) {
+    Write("void");
+  } else if (rt.types.size() == 1) {
     Write(rt.types[0]);
   } else {
-    Write("void");
+    Write("struct ", MangleMultivalueTypes(rt.types));
   }
 }
 
@@ -801,6 +821,30 @@ void CWriter::WriteSourceTop() {
   Write(s_source_declarations);
 }
 
+void CWriter::WriteMultivalueTypes() {
+  for (TypeEntry* type : module_->types) {
+    FuncType* func_type = cast<FuncType>(type);
+    Index num_results = func_type->GetNumResults();
+    if (num_results <= 1) {
+      continue;
+    }
+    std::string name = MangleMultivalueTypes(func_type->sig.result_types);
+    // these ifndefs are actually to support importing multiple modules
+    // incidentally they also mean we don't have to bother with deduplication
+    Write("#ifndef ", name, Newline());
+    Write("#define ", name, " ", name, Newline());
+    Write("struct ", name, " {", Newline());
+    for (Index i = 0; i < num_results; ++i) {
+      Type type = func_type->GetResultType(i);
+      Write("  ", type);
+      Writef(" %c%d;", MangleType(type), i);
+      Write(Newline());
+    }
+    Write("};", Newline(), "#endif  /* ", name, " */", Newline());
+  }
+}
+
+
 void CWriter::WriteFuncTypes() {
   Write(Newline());
   Writef("static u32 func_types[%" PRIzd "];", module_->types.size());
@@ -1033,9 +1077,9 @@ void CWriter::WriteDataInitializers() {
   }
   data_segment_index = 0;
   for (const DataSegment* data_segment : module_->data_segments) {
-    Write("memcpy(&(", ExternalRef(memory->name), ".data[");
+    Write("LOAD_DATA(", ExternalRef(memory->name), ", ");
     WriteInitExpr(data_segment->offset);
-    Write("]), data_segment_data_", data_segment_index, ", ",
+    Write(", data_segment_data_", data_segment_index, ", ",
           data_segment->data.size(), ");", Newline());
     ++data_segment_index;
   }
@@ -1208,9 +1252,20 @@ void CWriter::Write(const Func& func) {
   PushTypes(func.decl.sig.result_types);
   Write("FUNC_EPILOGUE;", Newline());
 
-  if (!func.decl.sig.result_types.empty()) {
-    // Return the top of the stack implicitly.
+  // Return the top of the stack implicitly.
+  Index num_results = func.GetNumResults();
+  if (num_results == 1) {
     Write("return ", StackVar(0), ";", Newline());
+  } else if (num_results >= 2) {
+    Write(OpenBrace());
+    Write(ResultType(func.decl.sig.result_types), " tmp;", Newline());
+    for (Index i = 0; i < num_results; ++i) {
+      Type type = func.GetResultType(i);
+      Writef("tmp.%c%d = ", MangleType(type), i);
+      Write(StackVar(num_results - i - 1), ";", Newline());
+    }
+    Write("return tmp;", Newline());
+    Write(CloseBrace(), Newline());
   }
 
   stream_ = c_stream_;
@@ -1320,8 +1375,10 @@ void CWriter::Write(const ExprList& exprs) {
       case ExprType::Block: {
         const Block& block = cast<BlockExpr>(&expr)->block;
         std::string label = DefineLocalScopeName(block.label);
+        DropTypes(block.decl.GetNumParams());
         size_t mark = MarkTypeStack();
         PushLabel(LabelType::Block, block.label, block.decl.sig);
+        PushTypes(block.decl.sig.param_types);
         Write(block.exprs, LabelDecl(label));
         ResetTypeStack(mark);
         PopLabel();
@@ -1361,8 +1418,11 @@ void CWriter::Write(const ExprList& exprs) {
         Index num_params = func.GetNumParams();
         Index num_results = func.GetNumResults();
         assert(type_stack_.size() >= num_params);
-        if (num_results > 0) {
-          assert(num_results == 1);
+        if (num_results > 1) {
+          Write(OpenBrace());
+          Write("struct ", MangleMultivalueTypes(func.decl.sig.result_types));
+          Write(" tmp = ");
+        } else if (num_results == 1) {
           Write(StackVar(num_params - 1, func.GetResultType(0)), " = ");
         }
 
@@ -1375,7 +1435,18 @@ void CWriter::Write(const ExprList& exprs) {
         }
         Write(");", Newline());
         DropTypes(num_params);
-        PushTypes(func.decl.sig.result_types);
+        if (num_results > 1) {
+          for (Index i = 0; i < num_results; ++i) {
+            Type type = func.GetResultType(i);
+            PushType(type);
+            Write(StackVar(0));
+            Writef(" = tmp.%c%d;", MangleType(type), i);
+            Write(Newline());
+          }
+          Write(CloseBrace(), Newline());
+        } else {
+          PushTypes(func.decl.sig.result_types);
+        }
         break;
       }
 
@@ -1384,8 +1455,11 @@ void CWriter::Write(const ExprList& exprs) {
         Index num_params = decl.GetNumParams();
         Index num_results = decl.GetNumResults();
         assert(type_stack_.size() > num_params);
-        if (num_results > 0) {
-          assert(num_results == 1);
+        if (num_results > 1) {
+          Write(OpenBrace());
+          Write("struct ", MangleMultivalueTypes(decl.sig.result_types));
+          Write(" tmp = ");
+        } else if (num_results == 1) {
           Write(StackVar(num_params, decl.GetResultType(0)), " = ");
         }
 
@@ -1403,7 +1477,18 @@ void CWriter::Write(const ExprList& exprs) {
         }
         Write(");", Newline());
         DropTypes(num_params + 1);
-        PushTypes(decl.sig.result_types);
+        if (num_results > 1) {
+          for (Index i = 0; i < num_results; ++i) {
+            Type type = decl.GetResultType(i);
+            PushType(type);
+            Write(StackVar(0));
+            Writef(" = tmp.%c%d;", MangleType(type), i);
+            Write(Newline());
+          }
+          Write(CloseBrace(), Newline());
+        } else {
+          PushTypes(decl.sig.result_types);
+        }
         break;
       }
 
@@ -1445,11 +1530,14 @@ void CWriter::Write(const ExprList& exprs) {
         Write("if (", StackVar(0), ") ", OpenBrace());
         DropTypes(1);
         std::string label = DefineLocalScopeName(if_.true_.label);
+        DropTypes(if_.true_.decl.GetNumParams());
         size_t mark = MarkTypeStack();
         PushLabel(LabelType::If, if_.true_.label, if_.true_.decl.sig);
+        PushTypes(if_.true_.decl.sig.param_types);
         Write(if_.true_.exprs, CloseBrace());
         if (!if_.false_.empty()) {
           ResetTypeStack(mark);
+          PushTypes(if_.true_.decl.sig.param_types);
           Write(" else ", OpenBrace(), if_.false_, CloseBrace());
         }
         ResetTypeStack(mark);
@@ -1488,8 +1576,10 @@ void CWriter::Write(const ExprList& exprs) {
         if (!block.exprs.empty()) {
           Write(DefineLocalScopeName(block.label), ": ");
           Indent();
+          DropTypes(block.decl.GetNumParams());
           size_t mark = MarkTypeStack();
           PushLabel(LabelType::Loop, block.label, block.decl.sig);
+          PushTypes(block.decl.sig.param_types);
           Write(Newline(), block.exprs);
           ResetTypeStack(mark);
           PopLabel();
@@ -1572,6 +1662,16 @@ void CWriter::Write(const ExprList& exprs) {
         break;
       }
 
+      case ExprType::SimdLoadLane: {
+        Write(*cast<SimdLoadLaneExpr>(&expr));
+        break;
+      }
+
+      case ExprType::SimdStoreLane: {
+        Write(*cast<SimdStoreLaneExpr>(&expr));
+        break;
+      }
+
       case ExprType::SimdShuffleOp: {
         Write(*cast<SimdShuffleOpExpr>(&expr));
         break;
@@ -1581,6 +1681,10 @@ void CWriter::Write(const ExprList& exprs) {
         Write(*cast<LoadSplatExpr>(&expr));
         break;
 
+      case ExprType::LoadZero:
+        Write(*cast<LoadZeroExpr>(&expr));
+        break;
+
       case ExprType::Unreachable:
         Write("UNREACHABLE;", Newline());
         return;
@@ -1592,12 +1696,12 @@ void CWriter::Write(const ExprList& exprs) {
       case ExprType::AtomicWait:
       case ExprType::AtomicFence:
       case ExprType::AtomicNotify:
-      case ExprType::BrOnExn:
       case ExprType::Rethrow:
       case ExprType::ReturnCall:
       case ExprType::ReturnCallIndirect:
       case ExprType::Throw:
       case ExprType::Try:
+      case ExprType::CallRef:
         UNIMPLEMENTED("...");
         break;
     }
@@ -1898,14 +2002,35 @@ void CWriter::Write(const ConvertExpr& expr) {
       break;
 
     case Opcode::I32TruncSatF32S:
+      WriteSimpleUnaryExpr(expr.opcode, "I32_TRUNC_SAT_S_F32");
+      break;
+
     case Opcode::I64TruncSatF32S:
+      WriteSimpleUnaryExpr(expr.opcode, "I64_TRUNC_SAT_S_F32");
+      break;
+
     case Opcode::I32TruncSatF64S:
+      WriteSimpleUnaryExpr(expr.opcode, "I32_TRUNC_SAT_S_F64");
+      break;
+
     case Opcode::I64TruncSatF64S:
+      WriteSimpleUnaryExpr(expr.opcode, "I64_TRUNC_SAT_S_F64");
+      break;
+
     case Opcode::I32TruncSatF32U:
+      WriteSimpleUnaryExpr(expr.opcode, "I32_TRUNC_SAT_U_F32");
+      break;
+
     case Opcode::I64TruncSatF32U:
+      WriteSimpleUnaryExpr(expr.opcode, "I64_TRUNC_SAT_U_F32");
+      break;
+
     case Opcode::I32TruncSatF64U:
+      WriteSimpleUnaryExpr(expr.opcode, "I32_TRUNC_SAT_U_F64");
+      break;
+
     case Opcode::I64TruncSatF64U:
-      UNIMPLEMENTED(expr.opcode.GetName());
+      WriteSimpleUnaryExpr(expr.opcode, "I64_TRUNC_SAT_U_F64");
       break;
 
     case Opcode::F32ConvertI32S:
@@ -1981,7 +2106,7 @@ void CWriter::Write(const LoadExpr& expr) {
     case Opcode::I32Load16S: func = "i32_load16_s"; break;
     case Opcode::I64Load16S: func = "i64_load16_s"; break;
     case Opcode::I32Load16U: func = "i32_load16_u"; break;
-    case Opcode::I64Load16U: func = "i32_load16_u"; break;
+    case Opcode::I64Load16U: func = "i64_load16_u"; break;
     case Opcode::I64Load32S: func = "i64_load32_s"; break;
     case Opcode::I64Load32U: func = "i64_load32_u"; break;
 
@@ -2109,11 +2234,23 @@ void CWriter::Write(const UnaryExpr& expr) {
       break;
 
     case Opcode::I32Extend8S:
+      WriteSimpleUnaryExpr(expr.opcode, "(u32)(s32)(s8)(u8)");
+      break;
+
     case Opcode::I32Extend16S:
+      WriteSimpleUnaryExpr(expr.opcode, "(u32)(s32)(s16)(u16)");
+      break;
+
     case Opcode::I64Extend8S:
+      WriteSimpleUnaryExpr(expr.opcode, "(u64)(s64)(s8)(u8)");
+      break;
+
     case Opcode::I64Extend16S:
+      WriteSimpleUnaryExpr(expr.opcode, "(u64)(s64)(s16)(u16)");
+      break;
+
     case Opcode::I64Extend32S:
-      UNIMPLEMENTED(expr.opcode.GetName());
+      WriteSimpleUnaryExpr(expr.opcode, "(u64)(s64)(s32)(u32)");
       break;
 
     default:
@@ -2172,6 +2309,14 @@ void CWriter::Write(const SimdLaneOpExpr& expr) {
   PushType(result_type);
 }
 
+void CWriter::Write(const SimdLoadLaneExpr& expr) {
+  UNIMPLEMENTED("SIMD support");
+}
+
+void CWriter::Write(const SimdStoreLaneExpr& expr) {
+  UNIMPLEMENTED("SIMD support");
+}
+
 void CWriter::Write(const SimdShuffleOpExpr& expr) {
   Type result_type = expr.opcode.GetResultType();
   Write(StackVar(1, result_type), " = ", expr.opcode.GetName(), "(",
@@ -2196,12 +2341,17 @@ void CWriter::Write(const LoadSplatExpr& expr) {
   PushType(result_type);
 }
 
+void CWriter::Write(const LoadZeroExpr& expr) {
+  UNIMPLEMENTED("SIMD support");
+}
+
 void CWriter::WriteCHeader() {
   stream_ = h_stream_;
   std::string guard = GenerateHeaderGuard();
   Write("#ifndef ", guard, Newline());
   Write("#define ", guard, Newline());
   Write(s_header_top);
+  WriteMultivalueTypes();
   WriteImports();
   WriteExports(WriteExportsKind::Declarations);
   Write(s_header_bottom);
