@@ -27,6 +27,7 @@ Result ExprVisitor::VisitExpr(Expr* root_expr) {
   state_stack_.clear();
   expr_stack_.clear();
   expr_iter_stack_.clear();
+  catch_index_stack_.clear();
 
   PushDefault(root_expr);
 
@@ -95,12 +96,23 @@ Result ExprVisitor::VisitExpr(Expr* root_expr) {
         if (iter != try_expr->block.exprs.end()) {
           PushDefault(&*iter++);
         } else {
-          CHECK_RESULT(delegate_->OnCatchExpr(try_expr));
           PopExprlist();
-          if (try_expr->catch_.empty()) {
-            CHECK_RESULT(delegate_->EndTryExpr(try_expr));
-          } else {
-            PushExprlist(State::Catch, expr, try_expr->catch_);
+          switch (try_expr->kind) {
+            case TryKind::Catch:
+              if (!try_expr->catches.empty()) {
+                Catch& catch_ = try_expr->catches[0];
+                CHECK_RESULT(delegate_->OnCatchExpr(try_expr, &catch_));
+                PushCatch(expr, 0, catch_.exprs);
+              } else {
+                CHECK_RESULT(delegate_->EndTryExpr(try_expr));
+              }
+              break;
+            case TryKind::Delegate:
+              CHECK_RESULT(delegate_->OnDelegateExpr(try_expr));
+              break;
+            case TryKind::Plain:
+              CHECK_RESULT(delegate_->EndTryExpr(try_expr));
+              break;
           }
         }
         break;
@@ -108,12 +120,20 @@ Result ExprVisitor::VisitExpr(Expr* root_expr) {
 
       case State::Catch: {
         auto try_expr = cast<TryExpr>(expr);
+        Index catch_index = catch_index_stack_.back();
         auto& iter = expr_iter_stack_.back();
-        if (iter != try_expr->catch_.end()) {
+        if (iter != try_expr->catches[catch_index].exprs.end()) {
           PushDefault(&*iter++);
         } else {
-          CHECK_RESULT(delegate_->EndTryExpr(try_expr));
-          PopExprlist();
+          PopCatch();
+          catch_index++;
+          if (catch_index < try_expr->catches.size()) {
+            Catch& catch_ = try_expr->catches[catch_index];
+            CHECK_RESULT(delegate_->OnCatchExpr(try_expr, &catch_));
+            PushCatch(expr, catch_index, catch_.exprs);
+          } else {
+            CHECK_RESULT(delegate_->EndTryExpr(try_expr));
+          }
         }
         break;
       }
@@ -183,10 +203,6 @@ Result ExprVisitor::HandleDefaultState(Expr* expr) {
       CHECK_RESULT(delegate_->OnBrIfExpr(cast<BrIfExpr>(expr)));
       break;
 
-    case ExprType::BrOnExn:
-      CHECK_RESULT(delegate_->OnBrOnExnExpr(cast<BrOnExnExpr>(expr)));
-      break;
-
     case ExprType::BrTable:
       CHECK_RESULT(delegate_->OnBrTableExpr(cast<BrTableExpr>(expr)));
       break;
@@ -199,6 +215,10 @@ Result ExprVisitor::HandleDefaultState(Expr* expr) {
       CHECK_RESULT(delegate_->OnCallIndirectExpr(cast<CallIndirectExpr>(expr)));
       break;
 
+    case ExprType::CallRef:
+      CHECK_RESULT(delegate_->OnCallRefExpr(cast<CallRefExpr>(expr)));
+      break;
+
     case ExprType::Compare:
       CHECK_RESULT(delegate_->OnCompareExpr(cast<CompareExpr>(expr)));
       break;
@@ -238,6 +258,10 @@ Result ExprVisitor::HandleDefaultState(Expr* expr) {
       CHECK_RESULT(delegate_->OnLoadSplatExpr(cast<LoadSplatExpr>(expr)));
       break;
 
+    case ExprType::LoadZero:
+      CHECK_RESULT(delegate_->OnLoadZeroExpr(cast<LoadZeroExpr>(expr)));
+      break;
+
     case ExprType::LocalGet:
       CHECK_RESULT(delegate_->OnLocalGetExpr(cast<LocalGetExpr>(expr)));
       break;
@@ -379,6 +403,17 @@ Result ExprVisitor::HandleDefaultState(Expr* expr) {
       break;
     }
 
+    case ExprType::SimdLoadLane: {
+      CHECK_RESULT(delegate_->OnSimdLoadLaneExpr(cast<SimdLoadLaneExpr>(expr)));
+      break;
+    }
+
+    case ExprType::SimdStoreLane: {
+      CHECK_RESULT(
+          delegate_->OnSimdStoreLaneExpr(cast<SimdStoreLaneExpr>(expr)));
+      break;
+    }
+
     case ExprType::SimdShuffleOp: {
       CHECK_RESULT(
           delegate_->OnSimdShuffleOpExpr(cast<SimdShuffleOpExpr>(expr)));
@@ -415,4 +450,20 @@ void ExprVisitor::PopExprlist() {
   expr_iter_stack_.pop_back();
 }
 
+void ExprVisitor::PushCatch(Expr* expr,
+                            Index catch_index,
+                            ExprList& expr_list) {
+  state_stack_.emplace_back(State::Catch);
+  expr_stack_.emplace_back(expr);
+  expr_iter_stack_.emplace_back(expr_list.begin());
+  catch_index_stack_.emplace_back(catch_index);
+}
+
+void ExprVisitor::PopCatch() {
+  state_stack_.pop_back();
+  expr_stack_.pop_back();
+  expr_iter_stack_.pop_back();
+  catch_index_stack_.pop_back();
+}
+
 }  // namespace wabt
