@@ -394,6 +394,16 @@ T WABT_VECTORCALL IntSubSat(T lhs, T rhs) {
   return Saturate<T, U>(lhs - rhs);
 }
 
+template <typename T>
+T WABT_VECTORCALL SaturatingRoundingQMul(T lhs, T rhs) {
+  constexpr int size_in_bits = sizeof(T) * 8;
+  int round_const = 1 << (size_in_bits - 2);
+  int64_t product = lhs * rhs;
+  product += round_const;
+  product >>= (size_in_bits - 1);
+  return Saturate<T, int64_t>(product);
+}
+
 }  // namespace interp
 }  // namespace wabt
 
