@@ -69,6 +69,13 @@ void Istream::Emit(Opcode::Enum op, u32 val1, u32 val2) {
   EmitInternal(val2);
 }
 
+void Istream::Emit(Opcode::Enum op, u32 val1, u32 val2, u8 val3) {
+  Emit(op);
+  EmitInternal(val1);
+  EmitInternal(val2);
+  EmitInternal(val3);
+}
+
 void Istream::EmitDropKeep(u32 drop, u32 keep) {
   if (drop > 0) {
     if (drop == 1 && keep == 0) {
@@ -161,14 +168,13 @@ Instr Istream::Read(Offset* offset) const {
     case Opcode::F64X2Sqrt:
     case Opcode::F64X2Trunc:
     case Opcode::I16X8AllTrue:
-    case Opcode::I16X8AnyTrue:
     case Opcode::I16X8Bitmask:
     case Opcode::I16X8Neg:
     case Opcode::I16X8Splat:
-    case Opcode::I16X8WidenHighI8X16S:
-    case Opcode::I16X8WidenHighI8X16U:
-    case Opcode::I16X8WidenLowI8X16S:
-    case Opcode::I16X8WidenLowI8X16U:
+    case Opcode::I16X8ExtendHighI8X16S:
+    case Opcode::I16X8ExtendHighI8X16U:
+    case Opcode::I16X8ExtendLowI8X16S:
+    case Opcode::I16X8ExtendLowI8X16U:
     case Opcode::I32Clz:
     case Opcode::I32Ctz:
     case Opcode::I32Eqz:
@@ -186,16 +192,15 @@ Instr Istream::Read(Offset* offset) const {
     case Opcode::I32TruncSatF64U:
     case Opcode::I32WrapI64:
     case Opcode::I32X4AllTrue:
-    case Opcode::I32X4AnyTrue:
     case Opcode::I32X4Bitmask:
     case Opcode::I32X4Neg:
     case Opcode::I32X4Splat:
     case Opcode::I32X4TruncSatF32X4S:
     case Opcode::I32X4TruncSatF32X4U:
-    case Opcode::I32X4WidenHighI16X8S:
-    case Opcode::I32X4WidenHighI16X8U:
-    case Opcode::I32X4WidenLowI16X8S:
-    case Opcode::I32X4WidenLowI16X8U:
+    case Opcode::I32X4ExtendHighI16X8S:
+    case Opcode::I32X4ExtendHighI16X8U:
+    case Opcode::I32X4ExtendLowI16X8S:
+    case Opcode::I32X4ExtendLowI16X8U:
     case Opcode::I64Clz:
     case Opcode::I64Ctz:
     case Opcode::I64Eqz:
@@ -215,17 +220,35 @@ Instr Istream::Read(Offset* offset) const {
     case Opcode::I64TruncSatF64S:
     case Opcode::I64TruncSatF64U:
     case Opcode::I64X2Neg:
+    case Opcode::I64X2AllTrue:
+    case Opcode::I64X2Bitmask:
+    case Opcode::I64X2ExtendLowI32X4S:
+    case Opcode::I64X2ExtendHighI32X4S:
+    case Opcode::I64X2ExtendLowI32X4U:
+    case Opcode::I64X2ExtendHighI32X4U:
     case Opcode::I64X2Splat:
     case Opcode::I8X16AllTrue:
-    case Opcode::I8X16AnyTrue:
     case Opcode::I8X16Bitmask:
     case Opcode::I8X16Neg:
+    case Opcode::I8X16Popcnt:
+    case Opcode::F32X4DemoteF64X2Zero:
+    case Opcode::F64X2PromoteLowF32X4:
+    case Opcode::I32X4TruncSatF64X2SZero:
+    case Opcode::I32X4TruncSatF64X2UZero:
+    case Opcode::F64X2ConvertLowI32X4S:
+    case Opcode::F64X2ConvertLowI32X4U:
     case Opcode::I8X16Splat:
     case Opcode::RefIsNull:
     case Opcode::V128Not:
+    case Opcode::V128AnyTrue:
     case Opcode::I8X16Abs:
     case Opcode::I16X8Abs:
     case Opcode::I32X4Abs:
+    case Opcode::I64X2Abs:
+    case Opcode::I16X8ExtaddPairwiseI8X16S:
+    case Opcode::I16X8ExtaddPairwiseI8X16U:
+    case Opcode::I32X4ExtaddPairwiseI16X8S:
+    case Opcode::I32X4ExtaddPairwiseI16X8U:
       // 0 immediates, 1 operand.
       instr.kind = InstrKind::Imm_0_Op_1;
       break;
@@ -284,6 +307,7 @@ Instr Istream::Read(Offset* offset) const {
     case Opcode::F64X2PMax:
     case Opcode::F64X2PMin:
     case Opcode::F64X2Sub:
+    case Opcode::I16X8Q15mulrSatS:
     case Opcode::I16X8Add:
     case Opcode::I16X8AddSatS:
     case Opcode::I16X8AddSatU:
@@ -311,6 +335,10 @@ Instr Istream::Read(Offset* offset) const {
     case Opcode::I16X8Sub:
     case Opcode::I16X8SubSatS:
     case Opcode::I16X8SubSatU:
+    case Opcode::I16X8ExtmulLowI8X16S:
+    case Opcode::I16X8ExtmulHighI8X16S:
+    case Opcode::I16X8ExtmulLowI8X16U:
+    case Opcode::I16X8ExtmulHighI8X16U:
     case Opcode::I32Add:
     case Opcode::I32And:
     case Opcode::I32DivS:
@@ -355,6 +383,11 @@ Instr Istream::Read(Offset* offset) const {
     case Opcode::I32X4ShrS:
     case Opcode::I32X4ShrU:
     case Opcode::I32X4Sub:
+    case Opcode::I32X4DotI16X8S:
+    case Opcode::I32X4ExtmulLowI16X8S:
+    case Opcode::I32X4ExtmulHighI16X8S:
+    case Opcode::I32X4ExtmulLowI16X8U:
+    case Opcode::I32X4ExtmulHighI16X8U:
     case Opcode::I32Xor:
     case Opcode::I64Add:
     case Opcode::I64And:
@@ -386,6 +419,16 @@ Instr Istream::Read(Offset* offset) const {
     case Opcode::I64X2ShrU:
     case Opcode::I64X2Sub:
     case Opcode::I64X2Mul:
+    case Opcode::I64X2Eq:
+    case Opcode::I64X2Ne:
+    case Opcode::I64X2LtS:
+    case Opcode::I64X2GtS:
+    case Opcode::I64X2LeS:
+    case Opcode::I64X2GeS:
+    case Opcode::I64X2ExtmulLowI32X4S:
+    case Opcode::I64X2ExtmulHighI32X4S:
+    case Opcode::I64X2ExtmulLowI32X4U:
+    case Opcode::I64X2ExtmulHighI32X4U:
     case Opcode::I64Xor:
     case Opcode::I8X16Add:
     case Opcode::I8X16AddSatS:
@@ -535,6 +578,8 @@ Instr Istream::Read(Offset* offset) const {
     case Opcode::V128Load64Splat:
     case Opcode::V128Load8Splat:
     case Opcode::V128Load:
+    case Opcode::V128Load32Zero:
+    case Opcode::V128Load64Zero:
       // Index + memory offset immediates, 1 operand.
       instr.kind = InstrKind::Imm_Index_Offset_Op_1;
       instr.imm_u32x2.fst = ReadAt<u32>(offset);
@@ -607,6 +652,21 @@ Instr Istream::Read(Offset* offset) const {
       instr.imm_u32x2.snd = ReadAt<u32>(offset);
       break;
 
+    case Opcode::V128Load8Lane:
+    case Opcode::V128Load16Lane:
+    case Opcode::V128Load32Lane:
+    case Opcode::V128Load64Lane:
+    case Opcode::V128Store8Lane:
+    case Opcode::V128Store16Lane:
+    case Opcode::V128Store32Lane:
+    case Opcode::V128Store64Lane:
+      // Index, memory offset, lane index immediates, 2 operands.
+      instr.kind = InstrKind::Imm_Index_Offset_Lane_Op_2;
+      instr.imm_u32x2_u8.fst = ReadAt<u32>(offset);
+      instr.imm_u32x2_u8.snd = ReadAt<u32>(offset);
+      instr.imm_u32x2_u8.idx = ReadAt<u8>(offset);
+      break;
+
     case Opcode::I32AtomicRmw16CmpxchgU:
     case Opcode::I32AtomicRmw8CmpxchgU:
     case Opcode::I32AtomicRmwCmpxchg:
@@ -691,9 +751,11 @@ Instr Istream::Read(Offset* offset) const {
       instr.imm_v128 = ReadAt<v128>(offset);
       break;
 
+    case Opcode::CallRef:
     case Opcode::Block:
-    case Opcode::BrOnExn:
     case Opcode::Catch:
+    case Opcode::CatchAll:
+    case Opcode::Delegate:
     case Opcode::Else:
     case Opcode::End:
     case Opcode::If:
@@ -828,6 +890,12 @@ Istream::Offset Istream::Trace(Stream* stream,
                      source->Pick(1, instr).c_str());
       break;
 
+    case InstrKind::Imm_Index_Offset_Lane_Op_2:
+      stream->Writef(" $%u:%s+$%u, %s (Lane imm: $%u)\n", instr.imm_u32x2_u8.fst,
+                     source->Pick(2, instr).c_str(), instr.imm_u32x2_u8.snd,
+                     source->Pick(1, instr).c_str(), instr.imm_u32x2_u8.idx);
+      break;
+
     case InstrKind::Imm_I32_Op_0:
       stream->Writef(" %u\n", instr.imm_u32);
       break;
