@@ -120,9 +120,6 @@ ModuleContext::Arities ModuleContext::GetExprArity(const Expr& expr) const {
       return { arity + 1, arity };
     }
 
-    case ExprType::BrOnExn:
-      return { 1, 1 };
-
     case ExprType::BrTable:
       return { GetLabelArity(cast<BrTableExpr>(&expr)->default_target) + 1, 1,
                true };
@@ -143,6 +140,11 @@ ModuleContext::Arities ModuleContext::GetExprArity(const Expr& expr) const {
                ci_expr->decl.GetNumResults() };
     }
 
+    case ExprType::CallRef: {
+      const Var& var = cast<CallRefExpr>(&expr)->function_type_index;
+      return { GetFuncParamCount(var) + 1, GetFuncResultCount(var) };
+    }
+
     case ExprType::ReturnCallIndirect: {
       const auto* rci_expr = cast<ReturnCallIndirectExpr>(&expr);
       return { rci_expr->decl.GetNumParams() + 1,
@@ -183,6 +185,7 @@ ModuleContext::Arities ModuleContext::GetExprArity(const Expr& expr) const {
     case ExprType::TableGet:
     case ExprType::RefIsNull:
     case ExprType::LoadSplat:
+    case ExprType::LoadZero:
       return { 1, 1 };
 
     case ExprType::Drop:
@@ -215,8 +218,8 @@ ModuleContext::Arities ModuleContext::GetExprArity(const Expr& expr) const {
     case ExprType::Throw: {
       auto throw_ = cast<ThrowExpr>(&expr);
       Index operand_count = 0;
-      if (Event* event = module.GetEvent(throw_->var)) {
-        operand_count = event->decl.sig.param_types.size();
+      if (Tag* tag = module.GetTag(throw_->var)) {
+        operand_count = tag->decl.sig.param_types.size();
       }
       return { operand_count, 0, true };
     }
@@ -256,6 +259,11 @@ ModuleContext::Arities ModuleContext::GetExprArity(const Expr& expr) const {
       }
     }
 
+    case ExprType::SimdLoadLane:
+    case ExprType::SimdStoreLane: {
+      return { 2, 1 };
+    }
+
     case ExprType::SimdShuffleOp:
       return { 2, 1 };
   }
