@@ -82,7 +82,7 @@ static void ParseOptions(int argc, char** argv) {
 
   parser.AddOption('v', "verbose", "Use multiple times for more info", []() {
     s_verbose++;
-    s_log_stream = FileStream::CreateStdout();
+    s_log_stream = FileStream::CreateStderr();
   });
   s_features.AddOptions(&parser);
   parser.AddOption('V', "value-stack-size", "SIZE",
